'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _cssClassCombiner = require('css-class-combiner');

var _cssClassCombiner2 = _interopRequireDefault(_cssClassCombiner);

var _AbstractSelection2 = require('./AbstractSelection');

var _AbstractSelection3 = _interopRequireDefault(_AbstractSelection2);

var _events = require('../utils/events');

var _cursors = require('../utils/cursors');

var _cursors2 = _interopRequireDefault(_cursors);

var _area = require('../utils/area');

var _resize = require('../core/resize');

var _resize2 = _interopRequireDefault(_resize);

var _drag = require('../core/drag');

var _drag2 = _interopRequireDefault(_drag);

var _sides = require('../utils/sides');

var sides = _interopRequireWildcard(_sides);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MIN_DIMENSION_SIZE_FOR_BIG_HANDLE = 40;
var BIG_HANDLE_SIZE = 8;
var SMALL_HANDLE_SIZE = 5;

var getHandleSize = function getHandleSize(dimensions) {
  var isTooSmall = Object.values(dimensions).some(function (d) {
    return d < MIN_DIMENSION_SIZE_FOR_BIG_HANDLE;
  });

  return isTooSmall ? SMALL_HANDLE_SIZE : BIG_HANDLE_SIZE;
};

var InteractiveSelection = function (_AbstractSelection) {
  _inherits(InteractiveSelection, _AbstractSelection);

  function InteractiveSelection(props) {
    _classCallCheck(this, InteractiveSelection);

    var _this = _possibleConstructorReturn(this, (InteractiveSelection.__proto__ || Object.getPrototypeOf(InteractiveSelection)).call(this, props));

    _this.containerParameters = props.containerParameters;
    _this.innerOffsets = {};
    _this.selectionEl = null;

    _this.state = {
      isFocused: false,
      isHovered: false,
      isDragging: false,
      area: props.area
    };

    _this.handleMouseDownOnSelection = _this.handleMouseDownOnSelection.bind(_this);
    _this.handleMouseUp = _this.handleMouseUp.bind(_this);
    _this.dragSelection = _this.dragSelection.bind(_this);
    _this.resizeSelection = _this.resizeSelection.bind(_this);

    _this.resizeCalculator = null;
    _this.dragCalculator = null;
    return _this;
  }

  _createClass(InteractiveSelection, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.selectionEl.addEventListener('mousedown', this.handleMouseDownOnSelection);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var area = nextProps.area,
          containerParameters = nextProps.containerParameters;


      this.resizeCalculator = (0, _resize2.default)(containerParameters);
      this.dragCalculator = (0, _drag2.default)(containerParameters);
      this.containerParameters = containerParameters;

      this.setState({ area: area });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.selectionEl.removeEventListener('mousedown', this.handleMouseDownOnSelection);
    }
  }, {
    key: 'getInnerOffsets',
    value: function getInnerOffsets(event) {
      var selectionOffsets = (0, _area.getSelectionOffsets)(this.selectionEl);

      return Object.freeze({
        left: event.clientX - selectionOffsets.left,
        top: (0, _events.getClientY)(event) - selectionOffsets.top
      });
    }
  }, {
    key: 'handleMouseUp',
    value: function handleMouseUp() {
      var isDragging = this.state.isDragging;


      if (isDragging) {
        this.stopDragSelection();
      } else {
        this.stopResizeSelection();
      }

      window.removeEventListener('mouseup', this.handleMouseUp);
    }
  }, {
    key: 'handleMouseDownOnSelection',
    value: function handleMouseDownOnSelection(event) {
      event.stopPropagation();

      if (this.props.frozen) {
        return;
      }

      this.selectionEl.focus();

      if (event.target === this.selectionEl) {
        this.startDrag(event);
      } else {
        this.startResize(event.target.dataset.side);
      }

      window.addEventListener('mouseup', this.handleMouseUp);
    }
  }, {
    key: 'dragSelection',
    value: function dragSelection(event) {
      var _this2 = this;

      event.stopPropagation();

      this.setState(function (state) {
        return {
          area: _this2.dragCalculator.calculate(event, state.area, _this2.innerOffsets)
        };
      });
    }
  }, {
    key: 'resizeSelection',
    value: function resizeSelection(event) {
      event.stopPropagation();

      var calculate = this.resizeCalculator.forSide(this.resizeSide);
      var area = calculate(event, this.state.area);

      this.setState({
        area: _extends({}, this.state.area, area)
      });
    }
  }, {
    key: 'stopDragSelection',
    value: function stopDragSelection() {
      this.setState({ isDragging: false });
      this.props.onAreaUpdate(this.state.area);

      (0, _cursors.setCursor)(_cursors2.default.DEFAULT);
      window.removeEventListener('mousemove', this.dragSelection);
    }
  }, {
    key: 'stopResizeSelection',
    value: function stopResizeSelection() {
      window.removeEventListener('mousemove', this.resizeSelection);
      this.props.onAreaUpdate(this.state.area);
    }
  }, {
    key: 'startResize',
    value: function startResize(resizeSide) {
      this.resizeSide = resizeSide;

      window.addEventListener('mousemove', this.resizeSelection);
    }
  }, {
    key: 'startDrag',
    value: function startDrag(event) {
      (0, _cursors.setCursor)(_cursors2.default.GRABBING);
      this.setState({ isDragging: true });

      this.innerOffsets = this.getInnerOffsets(event);

      window.addEventListener('mousemove', this.dragSelection);
    }
  }, {
    key: 'getClassName',
    value: function getClassName() {
      var _state = this.state,
          isHovered = _state.isHovered,
          isFocused = _state.isFocused,
          isDragging = _state.isDragging;
      var _props = this.props,
          className = _props.className,
          hoverClassName = _props.hoverClassName,
          focusClassName = _props.focusClassName,
          frozen = _props.frozen;


      return new _cssClassCombiner2.default('mr-selection').combine('mr-interactive-selection').combine(className).combineIf(isHovered, hoverClassName).combineIf(isFocused, focusClassName).combineIf(isDragging, 'mr-interactive-selection--drag').combineIf(frozen, 'mr-selection--transparent');
    }
  }, {
    key: 'getStyles',
    value: function getStyles() {
      return _extends({}, this.props.style, this.getPositionStyles(), {
        '--handle-size': getHandleSize(this.state.area.dimensions) + 'px'
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var styles = this.getStyles();
      var className = this.getClassName();

      return _react2.default.createElement(
        'div',
        {
          tabIndex: 0,
          role: 'button',
          ref: function ref(el) {
            return _this3.selectionEl = el;
          },
          className: className,
          style: styles,
          onFocus: this.handleFocus,
          onBlur: this.handleBlur,
          onMouseOver: this.handleMouseOver,
          onMouseOut: this.handleMouseOut,
          onDragStart: function onDragStart(e) {
            return e.preventDefault();
          }
        },
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-left',
          'data-side': sides.TOP_LEFT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-center',
          'data-side': sides.TOP
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--top-right',
          'data-side': sides.TOP_RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--right-center',
          'data-side': sides.RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-right',
          'data-side': sides.BOTTOM_RIGHT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-center',
          'data-side': sides.BOTTOM
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--bottom-left',
          'data-side': sides.BOTTOM_LEFT
        }),
        _react2.default.createElement('div', {
          className: 'mr-selection__handle mr-selection__handle--left-center',
          'data-side': sides.LEFT
        })
      );
    }
  }]);

  return InteractiveSelection;
}(_AbstractSelection3.default);

InteractiveSelection.propTypes = {
  frozen: _propTypes2.default.bool,
  onAreaUpdate: _propTypes2.default.func
};

InteractiveSelection.defaultProps = {
  frozen: false,
  onAreaUpdate: null
};

exports.default = InteractiveSelection;