'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _container = require('./container');

var _container2 = _interopRequireDefault(_container);

var _selection = require('./selection');

var _selection2 = _interopRequireDefault(_selection);

var _events = require('../utils/events');

var _sides = require('../utils/sides');

var sides = _interopRequireWildcard(_sides);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var MIN_WIDTH = 10;

var replaceSecondWith = function replaceSecondWith(replacement) {
  return function (arg, index) {
    return index !== 1 ? arg : replacement;
  };
};

function ResizeCalculator(containerParams) {
  var container = (0, _container2.default)(containerParams);

  var calculateRightResize = function calculateRightResize(event, selectionParams) {
    var selection = (0, _selection2.default)(selectionParams);

    var innerX = selection.x() - container.paddingLeft();
    var boxRightBorder = innerX + selection.width();
    var xWithinContainer = event.clientX - container.offsetLeft();
    var tooCloseToLeft = xWithinContainer - innerX <= MIN_WIDTH;

    if (tooCloseToLeft) {
      return selectionParams;
    }

    var nextSelectionWidth = container.width() - innerX;

    if (xWithinContainer <= container.width()) {
      var travelDistance = xWithinContainer - boxRightBorder;

      nextSelectionWidth = selection.width() + travelDistance;
    }

    return Object.freeze({
      dimensions: {
        width: nextSelectionWidth,
        height: selection.height()
      },
      coordinates: selectionParams.coordinates
    });
  };

  var calculateLeftResize = function calculateLeftResize(event, selectionParams) {
    var selection = (0, _selection2.default)(selectionParams);

    var innerX = selection.x() - container.paddingLeft();
    var boxRightBorder = innerX + selection.width();
    var xWithinContainer = event.clientX - container.offsetLeft();
    var isTooCloseToRight = boxRightBorder - xWithinContainer <= MIN_WIDTH;

    if (isTooCloseToRight) {
      return selectionParams;
    }

    var nextX = 0;
    var nextSelectionWidth = boxRightBorder;

    if (xWithinContainer > 0) {
      var travelDistance = xWithinContainer - innerX;

      nextX = innerX + travelDistance;
      nextSelectionWidth = selection.width() - travelDistance;
    }

    return Object.freeze({
      dimensions: {
        width: nextSelectionWidth,
        height: selection.height()
      },
      coordinates: {
        x: nextX + container.paddingLeft(),
        y: selection.y()
      }
    });
  };

  var calculateBottomResize = function calculateBottomResize(event, selectionParams) {
    var selection = (0, _selection2.default)(selectionParams);

    var innerY = selection.y() - container.paddingTop();
    var yWithinContainer = (0, _events.getClientY)(event) - container.offsetTop();
    var boxBottomBorder = innerY + selection.height();
    var tooCloseToTop = yWithinContainer - innerY <= MIN_WIDTH;

    if (tooCloseToTop) {
      return selectionParams;
    }

    var nextSelectionHeight = void 0;

    if (yWithinContainer > container.height()) {
      nextSelectionHeight = container.height() - innerY;
    } else {
      var travelDistance = yWithinContainer - boxBottomBorder;

      nextSelectionHeight = selection.height() + travelDistance;
    }

    return Object.freeze({
      coordinates: selectionParams.coordinates,
      dimensions: {
        height: nextSelectionHeight,
        width: selection.width()
      }
    });
  };

  var calculateTopResize = function calculateTopResize(event, selectionParams) {
    var selection = (0, _selection2.default)(selectionParams);

    var innerY = selection.y() - container.paddingTop();
    var selectionBottomBorder = innerY + selection.height();
    var yWithinContainer = (0, _events.getClientY)(event) - container.offsetTop();
    var tooCloseToBottom = selectionBottomBorder - yWithinContainer <= MIN_WIDTH;

    if (tooCloseToBottom) {
      return selectionParams;
    }

    var nextY = 0;
    var nextSelectionHeight = selectionBottomBorder;

    if (yWithinContainer > 0) {
      var travelDistance = innerY - yWithinContainer;

      nextSelectionHeight = selection.height() + travelDistance;
      nextY = innerY - travelDistance;
    }

    return Object.freeze({
      coordinates: {
        x: selection.x(),
        y: nextY + container.paddingTop()
      },
      dimensions: {
        width: selection.width(),
        height: nextSelectionHeight
      }
    });
  };

  var calculateBottomRightResize = function calculateBottomRightResize() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var nextSelectionParams = calculateBottomResize.apply(undefined, args);
    var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

    return calculateRightResize.apply(undefined, _toConsumableArray(nextArgs));
  };

  var calculateBottomLeftResize = function calculateBottomLeftResize() {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    var nextSelectionParams = calculateBottomResize.apply(undefined, args);
    var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

    return calculateLeftResize.apply(undefined, _toConsumableArray(nextArgs));
  };

  var calculateTopLeftResize = function calculateTopLeftResize() {
    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var nextSelectionParams = calculateTopResize.apply(undefined, args);
    var nextArgs = args.map(replaceSecondWith(nextSelectionParams));

    return calculateLeftResize.apply(undefined, _toConsumableArray(nextArgs));
  };

  var calculateTopRightResize = function calculateTopRightResize() {
    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      args[_key4] = arguments[_key4];
    }

    var updatedSelectionParams = calculateTopResize.apply(undefined, args);
    var nextArgs = args.map(replaceSecondWith(updatedSelectionParams));

    return calculateRightResize.apply(undefined, _toConsumableArray(nextArgs));
  };

  var forSide = function forSide(side) {
    var _sides$TOP$sides$BOTT;

    return (_sides$TOP$sides$BOTT = {}, _defineProperty(_sides$TOP$sides$BOTT, sides.TOP, calculateTopResize), _defineProperty(_sides$TOP$sides$BOTT, sides.BOTTOM, calculateBottomResize), _defineProperty(_sides$TOP$sides$BOTT, sides.RIGHT, calculateRightResize), _defineProperty(_sides$TOP$sides$BOTT, sides.LEFT, calculateLeftResize), _defineProperty(_sides$TOP$sides$BOTT, sides.BOTTOM_RIGHT, calculateBottomRightResize), _defineProperty(_sides$TOP$sides$BOTT, sides.BOTTOM_LEFT, calculateBottomLeftResize), _defineProperty(_sides$TOP$sides$BOTT, sides.TOP_RIGHT, calculateTopRightResize), _defineProperty(_sides$TOP$sides$BOTT, sides.TOP_LEFT, calculateTopLeftResize), _sides$TOP$sides$BOTT)[side];
  };

  return Object.freeze({ forSide: forSide });
}

exports.default = ResizeCalculator;